use 5.14.0;
use strict;
use Test;

#BEGIN { plan tests => 5, todo => [1,2,3,4,5] }
BEGIN { plan tests => 1 }

ok(1)

#chdir ("t");
#mkdir("_tmp");
#chdir ("_tmp");
#system("../../bin/dapper init");

#ok(-f "_config.yml");
#ok(-d "_source");
#ok(-f "_source/index.md");
#ok(-d "_layout");
#ok(-f "_layout/index.html");

#system("rm _config.yml");
#system("rm -rf _source");
#system("rm -rf _layout");
#chdir("..");
#system("rm -rf _tmp");

